@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.186 2011/03/02 09:58:29 tom Exp $
+ * $Id: dialog.c,v 1.202 2012/07/01 20:20:39 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2010,2011	Thomas E. Dickey
+ *  Copyright 2000-2011,2012	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -57,6 +57,7 @@ typedef enum {
     ,o_cr_wrap
     ,o_create_rc
     ,o_date_format
+    ,o_default_button
     ,o_default_item
     ,o_defaultno
     ,o_dselect
@@ -71,7 +72,9 @@ typedef enum {
     ,o_gauge
     ,o_help
     ,o_help_button
+    ,o_help_file
     ,o_help_label
+    ,o_help_line
     ,o_help_status
     ,o_icon
     ,o_ignore
@@ -96,6 +99,7 @@ typedef enum {
     ,o_no_label
     ,o_no_lines
     ,o_no_mouse
+    ,o_no_nl_expand
     ,o_no_shadow
     ,o_nocancel
     ,o_noitem
@@ -137,6 +141,7 @@ typedef enum {
     ,o_title
     ,o_trim
     ,o_under_mouse
+    ,o_version
     ,o_visit_items
     ,o_wmclass
     ,o_yes_label
@@ -197,6 +202,7 @@ static const Options options[] = {
     { "cr-wrap",	o_cr_wrap,		1, "" },
     { "create-rc",	o_create_rc,		1, NULL },
     { "date-format",	o_date_format,		1, "<str>" },
+    { "default-button",	o_default_button,	1, "<str>" },
     { "default-item",	o_default_item,		1, "<str>" },
     { "defaultno",	o_defaultno,		1, "" },
     { "dselect",	o_dselect,		2, "<directory> <height> <width>" },
@@ -215,6 +221,8 @@ static const Options options[] = {
     { "help-button",	o_help_button,		1, "" },
     { "help-label",	o_help_label,		1, "<str>" },
     { "help-status",	o_help_status,		1, "" },
+    { "hfile",		o_help_file,		1, "<str>" },
+    { "hline",		o_help_line,		1, "<str>" },
     { "icon",		o_icon,			1, NULL },
     { "ignore",		o_ignore,		1, "" },
     { "infobox",	o_infobox,		2, "<text> <height> <width>" },
@@ -239,6 +247,7 @@ static const Options options[] = {
     { "no-label",	o_no_label,		1, "<str>" },
     { "no-lines",	o_no_lines, 		1, "" },
     { "no-mouse",	o_no_mouse,		1, "" },
+    { "no-nl-expand",	o_no_nl_expand,		1, "" },
     { "no-ok",		o_nook,			1, "" },
     { "no-shadow",	o_no_shadow,		1, "" },
     { "nocancel",	o_nocancel,		1, NULL }, /* see --no-cancel */
@@ -281,7 +290,7 @@ static const Options options[] = {
     { "title",		o_title,		1, "<title>" },
     { "trim",		o_trim,			1, "" },
     { "under-mouse", 	o_under_mouse,		1, NULL },
-    { "version",	o_print_version,	5, "" },
+    { "version",	o_version,		5, "" },
     { "visit-items", 	o_visit_items,		1, "" },
     { "wmclass",	o_wmclass,		1, NULL },
     { "yes-label",	o_yes_label,		1, "<str>" },
@@ -424,6 +433,12 @@ unescape_argv(int *argcp, char ***argvp)
     dialog_argv = (*argvp);
 }
 
+#define OptionChars "\
+0123456789\
+-\
+abcdefghijklmnopqrstuvwxyz\
+"
+
 /*
  * Check if the given string from main's argv is an option.
  */
@@ -442,7 +457,11 @@ isOption(const char *arg)
 		}
 	    }
 	} else if (!strncmp(arg, "--", (size_t) 2) && isalpha(UCH(arg[2]))) {
-	    result = TRUE;
+	    if (strlen(arg) == strspn(arg, OptionChars)) {
+		result = TRUE;
+	    } else {
+		dlg_exiterr("Invalid option \"%s\"", arg);
+	    }
 	}
     }
     return result;
@@ -452,17 +471,19 @@ static eOptions
 lookupOption(const char *name, int pass)
 {
     unsigned n;
+    eOptions result = o_unknown;
 
     if (isOption(name)) {
 	name += 2;
 	for (n = 0; n < sizeof(options) / sizeof(options[0]); n++) {
 	    if ((pass & options[n].pass) != 0
 		&& !strcmp(name, options[n].name)) {
-		return options[n].code;
+		result = options[n].code;
+		break;
 	    }
 	}
     }
-    return o_unknown;
+    return result;
 }
 
 static void
@@ -678,9 +699,7 @@ call_checklist(CALLARGS)
 {
     int tags = howmany_tags(av + 5, CHECKBOX_TAGS);
     int code;
-    bool save_quoted = dialog_vars.quoted;
 
-    dialog_vars.quoted = !dialog_vars.separate_output;
     *offset_add = 5 + tags * CHECKBOX_TAGS;
     code = dialog_checklist(t,
 			    av[1],
@@ -688,7 +707,6 @@ call_checklist(CALLARGS)
 			    numeric_arg(av, 3),
 			    numeric_arg(av, 4),
 			    tags, av + 5, FLAG_CHECK);
-    dialog_vars.quoted = save_quoted;
     return code;
 }
 
@@ -872,6 +890,7 @@ call_mixed_gauge(CALLARGS)
 }
 #endif
 
+#ifdef HAVE_DLG_GAUGE
 static int
 call_prgbox(CALLARGS)
 {
@@ -891,6 +910,7 @@ call_prgbox(CALLARGS)
 			    numeric_arg(av, 2),
 			    numeric_arg(av, 3), TRUE));
 }
+#endif
 
 #ifdef HAVE_DLG_GAUGE
 static int
@@ -1043,6 +1063,43 @@ optionValue(char **argv, int *num)
     return result;
 }
 
+/* Return exit-code for a named button */
+static int
+button_code(const char *name)
+{
+    /* *INDENT-OFF* */
+    static struct {
+	const char *name;
+	int code;
+    } table[] = {
+	{ "ok",	    DLG_EXIT_OK },
+	{ "yes",    DLG_EXIT_OK },
+	{ "cancel", DLG_EXIT_CANCEL },
+	{ "no",	    DLG_EXIT_CANCEL },
+	{ "help",   DLG_EXIT_HELP },
+	{ "extra",  DLG_EXIT_EXTRA },
+    };
+    /* *INDENT-ON* */
+
+    int code = DLG_EXIT_ERROR;
+    size_t i;
+
+    for (i = 0; i < (sizeof(table) / sizeof(table[0])); i++) {
+	if (!dlg_strcmp(name, table[i].name)) {
+	    code = table[i].code;
+	    break;
+	}
+    }
+
+    if (code == DLG_EXIT_ERROR) {
+	char temp[80];
+	sprintf(temp, "Button name \"%.20s\" unknown", name);
+	Usage(temp);
+    }
+
+    return code;
+}
+
 /*
  * Print parts of a message
  */
@@ -1088,6 +1145,15 @@ compare_opts(const void *a, const void *b)
     return strcmp((*p)->name, (*q)->name);
 }
 
+/*
+ * Print program's version.
+ */
+static void
+PrintVersion(FILE *fp)
+{
+    fprintf(fp, "Version: %s\n", dialog_version());
+}
+
 /*
  * Print program help-message
  */
@@ -1097,7 +1163,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2008,2011 Thomas E. Dickey",
+	"Copyright 2000-2011,2012 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1122,6 +1188,9 @@ Help(void)
     size_t j, k;
     const Options **opts;
 
+    end_dialog();
+    dialog_state.output = stdout;
+
     opts = dlg_calloc(const Options *, limit);
     assert_ptr(opts, "Help");
     for (j = 0; j < limit; ++j) {
@@ -1158,6 +1227,29 @@ Help(void)
     dlg_exit(DLG_EXIT_OK);
 }
 
+#ifdef HAVE_DLG_TRACE
+/*
+ * Only the first call to dlg_trace will open a trace file.  But each time
+ * --trace is parsed, we show the whole parameter list as it is at that moment,
+ * counting discarded parameters.  The only way to capture the whole parameter
+ * list is if --trace is the first option.
+ */
+static void
+process_trace_option(char **argv, int *offset)
+{
+    int j;
+
+    if (dialog_state.trace_output == 0)
+	dlg_trace(optionString(argv, offset));
+
+    dlg_trace_msg("# Parameters:\n");
+    for (j = 0; argv[j] != 0; ++j) {
+	dlg_trace_msg("# argv[%d] = %s\n", j, argv[j]);
+    }
+    *offset += 1;
+}
+#endif
+
 /*
  * "Common" options apply to all widgets more/less.  Most of the common options
  * set values in dialog_vars, a few set dialog_state and a couple write to the
@@ -1166,12 +1258,12 @@ Help(void)
 static int
 process_common_options(int argc, char **argv, int offset, bool output)
 {
-#ifdef HAVE_DLG_TRACE
-    int n;
-#endif
     bool done = FALSE;
 
+    dlg_trace_msg("# process_common_options, offset %d\n", offset);
+
     while (offset < argc && !done) {	/* Common options */
+	dlg_trace_msg("#\targv[%d] = %s\n", offset, argv[offset]);
 	switch (lookupOption(argv[offset], 1)) {
 	case o_title:
 	    dialog_vars.title = optionString(argv, &offset);
@@ -1191,6 +1283,9 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	case o_cr_wrap:
 	    dialog_vars.cr_wrap = TRUE;
 	    break;
+	case o_no_nl_expand:
+	    dialog_vars.no_nl_expand = TRUE;
+	    break;
 	case o_no_collapse:
 	    dialog_vars.nocollapse = TRUE;
 	    break;
@@ -1226,6 +1321,11 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    break;
 	case o_defaultno:
 	    dialog_vars.defaultno = TRUE;
+	    dialog_vars.default_button = DLG_EXIT_CANCEL;
+	    break;
+	case o_default_button:
+	    dialog_vars.default_button = button_code(optionString(argv, &offset));
+	    dialog_vars.defaultno = dialog_vars.default_button == DLG_EXIT_CANCEL;
 	    break;
 	case o_default_item:
 	    dialog_vars.default_item = optionString(argv, &offset);
@@ -1236,6 +1336,12 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	case o_item_help:
 	    dialog_vars.item_help = TRUE;
 	    break;
+	case o_help_line:
+	    dialog_vars.help_line = optionString(argv, &offset);
+	    break;
+	case o_help_file:
+	    dialog_vars.help_file = optionString(argv, &offset);
+	    break;
 	case o_help_button:
 	    dialog_vars.help_button = TRUE;
 	    break;
@@ -1274,7 +1380,7 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    break;
 	case o_print_version:
 	    if (output) {
-		fprintf(stdout, "Version: %s\n", dialog_version());
+		PrintVersion(dialog_state.output);
 	    }
 	    break;
 	case o_separator:
@@ -1356,6 +1462,7 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    break;
 	case o_no_mouse:
 	    dialog_state.no_mouse = TRUE;
+	    mouse_close();
 	    break;
 	case o_noitem:
 	case o_fullbutton:
@@ -1385,10 +1492,7 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    break;
 #ifdef HAVE_DLG_TRACE
 	case o_trace:
-	    dlg_trace(optionString(argv, &offset));
-	    for (n = 0; argv[n] != 0; ++n) {
-		dlg_trace_msg("argv[%d] = %s\n", n, argv[n]);
-	    }
+	    process_trace_option(argv, &offset);
 	    break;
 #endif
 	}
@@ -1409,12 +1513,16 @@ init_result(char *buffer)
     static char **special_argv = 0;
     static int special_argc = 0;
 
+    dlg_trace_msg("# init_result\n");
+
     /* clear everything we do not save for the next widget */
     memset(&dialog_vars, 0, sizeof(dialog_vars));
 
     dialog_vars.input_result = buffer;
     dialog_vars.input_result[0] = '\0';
 
+    dialog_vars.default_button = -1;
+
     /*
      * The first time this is called, check for common options given by an
      * environment variable.
@@ -1427,12 +1535,20 @@ init_result(char *buffer)
 	    special_argv = dlg_string_to_argv(env);
 	    special_argc = dlg_count_argv(special_argv);
 	}
+	first = FALSE;
     }
+
+    /*
+     * If we are not checking memory leaks, just do the parse of the
+     * environment once.
+     */
     if (special_argv != 0) {
 	process_common_options(special_argc, special_argv, 0, FALSE);
 #ifdef NO_LEAKS
 	free(special_argv[0]);
 	free(special_argv);
+	special_argv = 0;
+	special_argc = 0;
 	first = TRUE;
 #endif
     }
@@ -1474,6 +1590,9 @@ main(int argc, char *argv[])
      * that.  We can only write to one of them.  If --stdout is used, that
      * can interfere with initializing the curses library, so we want to
      * know explicitly if it is used.
+     *
+     * Also, look for any --version or --help message, processing those
+     * immediately.
      */
     while (offset < argc) {
 	int base = offset;
@@ -1497,10 +1616,36 @@ main(int argc, char *argv[])
 	case o_keep_tite:
 	    keep_tite = TRUE;
 	    break;
+	case o_version:
+	    dialog_state.output = stdout;
+	    PrintVersion(dialog_state.output);
+	    exit(DLG_EXIT_OK);
+	    break;
+	case o_help:
+	    Help();
+	    break;
+#ifdef HAVE_DLG_TRACE
+	case o_trace:
+	    /*
+	     * Process/remove the --trace option if it is the first option.
+	     * Otherwise, process it in more/less expected order as a
+	     * "common" option.
+	     */
+	    if (base == 1) {
+		process_trace_option(argv, &offset);
+		break;
+	    } else {
+		++offset;
+		continue;
+	    }
+#endif
 	default:
 	    ++offset;
 	    continue;
 	}
+	dlg_trace_msg("# discarding %d parameters starting with argv[%d] (%s)\n",
+		      1 + offset - base, base,
+		      argv[base]);
 	for (j = base; j < argc; ++j) {
 	    dialog_argv[j] = dialog_argv[j + 1 + (offset - base)];
 	    if (dialog_opts != 0)
@@ -1512,7 +1657,11 @@ main(int argc, char *argv[])
     offset = 1;
     init_result(my_buffer);
 
-    if (argc == 2) {		/* if we don't want clear screen */
+    /*
+     * Dialog's output may be redirected (see above).  Handle the special
+     * case of options that only report information without interaction.
+     */
+    if (argc == 2) {
 	switch (lookupOption(argv[1], 7)) {
 	case o_print_maxsize:
 	    (void) initscr();
@@ -1521,7 +1670,7 @@ main(int argc, char *argv[])
 	    fprintf(dialog_state.output, "MaxSize: %d, %d\n", SLINES, SCOLS);
 	    break;
 	case o_print_version:
-	    fprintf(stdout, "Version: %s\n", dialog_version());
+	    PrintVersion(dialog_state.output);
 	    break;
 	case o_clear:
 	    initscr();
@@ -1531,8 +1680,6 @@ main(int argc, char *argv[])
 	case o_ignore:
 	    break;
 	default:
-	case o_help:
-	    dialog_state.output = stdout;
 	    Help();
 	    break;
 	}
@@ -1621,6 +1768,8 @@ main(int argc, char *argv[])
 	    case o_unknown:
 	    case o_title:
 	    case o_backtitle:
+	    case o_help_line:
+	    case o_help_file:
 		break;
 	    default:
 		if (argv[j] != 0) {
@@ -1633,6 +1782,7 @@ main(int argc, char *argv[])
 	retval = show_result((*(modePtr->jumper)) (dialog_vars.title,
 						   argv + offset,
 						   &offset_add));
+	dlg_trace_msg("# widget returns %d\n", retval);
 	offset += offset_add;
 
 	if (dialog_vars.input_result != my_buffer) {
