@@ -1,9 +1,9 @@
 /*
- *  $Id: dialog.h,v 1.223 2011/03/02 10:04:09 tom Exp $
+ *  $Id: dialog.h,v 1.245 2012/07/01 18:44:03 tom Exp $
  *
  *  dialog.h -- common declarations for all dialog modules
  *
- *  Copyright 2000-2010,2011	Thomas E. Dickey
+ *  Copyright 2000-2011,2012	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -44,7 +44,10 @@
 #include <signal.h>	/* fork() etc. */
 #include <math.h>	/* sqrt() */
 
-#undef ERR		/* header conflict with Solaris xpg4 */
+/* header conflict with Solaris xpg4 versus <sys/regset.h> */
+#if defined(ERR) && (ERR == 13)
+#undef ERR
+#endif
 
 #if defined(HAVE_NCURSESW_NCURSES_H)
 #include <ncursesw/ncurses.h>
@@ -56,11 +59,9 @@
 #include <ncurses.h>
 #else
 #include <curses.h>
-#endif
-
-/* most curses.h headers include this, some do not */
 #if defined(HAVE_UNCTRL_H)
-#include <unctrl.h>
+#include <unctrl.h> /* most curses.h headers include this, some do not */
+#endif
 #endif
 
 /* Solaris xpg4 renames these */
@@ -94,6 +95,10 @@
 #define _(s) s
 #endif
 
+#ifndef GCC_PRINTFLIKE
+#define GCC_PRINTFLIKE(fmt,var) /*nothing*/
+#endif
+
 #ifndef GCC_NORETURN
 #define GCC_NORETURN /*nothing*/
 #endif
@@ -140,6 +145,7 @@
 
 #define DLG_CTRL(n)	((n) & 0x1f)	/* CTRL is preferred, but conflicts */
 
+#define CHR_HELP	DLG_CTRL('E')
 #define CHR_BACKSPACE	DLG_CTRL('H')
 #define CHR_REPAINT	DLG_CTRL('L')
 #define CHR_KILL	DLG_CTRL('U')
@@ -229,6 +235,11 @@
 #define getparyx(win,y,x)	(y = (win)?(win)->_pary:ERR, x = (win)?(win)->_parx:ERR)
 #endif
 
+#if !defined(HAVE_WGETPARENT) && defined(HAVE_WINDOW__PARENT)
+#undef  wgetparent
+#define wgetparent(win)		((win) ? (win)->_parent : 0)
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -270,6 +281,12 @@ extern int dlg_getparx(WINDOW * /*win*/);
 extern int dlg_getpary(WINDOW * /*win*/);
 #endif
 
+#if !(defined(HAVE_WGETPARENT) && defined(HAVE_WINDOW__PARENT))
+#undef wgetparent
+#define wgetparent(win) dlg_wgetparent(win)
+extern WINDOW * dlg_wgetparent(WINDOW * /*win*/);
+#endif
+
 /*
  * This is a list of "old" names, which should be helpful in updating
  * applications that use libdialog.  Starting with 2003/11/26, all exported
@@ -353,6 +370,10 @@ extern int dlg_getpary(WINDOW * /*win*/);
 #define form_text_attr                DIALOG_ATR(31)
 #define form_item_readonly_attr       DIALOG_ATR(32)
 #define gauge_attr                    DIALOG_ATR(33)
+#define border2_attr                  DIALOG_ATR(34)
+#define inputbox_border2_attr         DIALOG_ATR(35)
+#define searchbox_border2_attr        DIALOG_ATR(36)
+#define menubox_border2_attr          DIALOG_ATR(37)
 
 #define DLGK_max (KEY_MAX + 256)
 
@@ -392,6 +413,7 @@ typedef struct {
     DIALOG_CALLBACK *getc_callbacks;
     DIALOG_CALLBACK *getc_redirect;
     DIALOG_WINDOWS *all_windows;
+    DIALOG_WINDOWS *all_subwindows;
     FILE *output;		/* option "--output-fd fd" */
     FILE *pipe_input;		/* used for gauge widget */
     FILE *screen_output;	/* newterm(), etc. */
@@ -476,6 +498,13 @@ typedef struct {
     /* 1.1-20100118 */
     char *date_format;		/* option "--date-format" */
     char *time_format;		/* option "--time-format" */
+    /* 1.1-20110629 */
+    char *help_line;		/* option "--hline" */
+    char *help_file;		/* option "--hfile" */
+    bool in_helpfile;		/* flag to prevent recursion in --hfile */
+    bool no_nl_expand;		/* option "--no-nl-expand" */
+    /* 1.1-20120701 */
+    int default_button;		/* option "--default-button" (exit code) */
 } DIALOG_VARS;
 
 #define USE_ITEM_HELP(s)        (dialog_vars.item_help && (s) != 0)
@@ -530,6 +559,7 @@ extern int dialog_editbox(const char * /*title*/, const char * /*file*/, int /*h
 extern int dialog_form(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, int /*form_height*/, int /*item_no*/, char ** /*items*/);
 extern int dialog_fselect(const char * /*title*/, const char * /*path*/, int /*height*/, int /*width*/);
 extern int dialog_gauge(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, int /*percent*/);
+extern int dialog_helpfile(const char * /*title*/, const char * /*file*/, int /*height*/, int /*width*/);
 extern int dialog_inputbox(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, const char * /*init*/, const int /*password*/);
 extern int dialog_menu(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, int /*menu_height*/, int /*item_no*/, char ** /*items*/);
 extern int dialog_mixedform(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, int /*form_height*/, int /*item_no*/, char ** /*items*/);
@@ -584,6 +614,7 @@ extern int dlg_eat_argv(int * /* argcp */, char *** /* argvp */, int /* start */
 /* arrows.c */
 extern void dlg_draw_arrows(WINDOW * /*dialog*/, int /*top_arrow*/, int /*bottom_arrow*/, int /*x*/, int /*top*/, int /*bottom*/);
 extern void dlg_draw_arrows2(WINDOW * /*dialog*/, int /*top_arrow*/, int /*bottom_arrow*/, int /*x*/, int /*top*/, int /*bottom*/, chtype /*attr*/, chtype /*borderattr*/);
+extern void dlg_draw_helpline(WINDOW * /*dialog*/, bool /*decorations*/);
 extern void dlg_draw_scrollbar(WINDOW * /*dialog*/, long /* first_data */, long /* this_data */, long /* next_data */, long /* total_data */, int /* left */, int /* right */, int /*top*/, int /*bottom*/, chtype /*attr*/, chtype /*borderattr*/);
 
 /* buttons.c */
@@ -612,13 +643,18 @@ extern void dlg_align_columns(char ** /* target */, int  /* per_row */, int /* n
 extern void dlg_free_columns(char ** /* target */, int  /* per_row */, int /* num_rows */);
 
 /* editbox.c */
-extern int dlg_editbox(const char */*title*/, char ***/*list*/, int */*rows*/, int /*height*/, int /*width*/);
+extern int dlg_editbox(const char * /*title*/, char *** /*list*/, int * /*rows*/, int /*height*/, int /*width*/);
 
 /* formbox.c */
 extern int dlg_default_formitem(DIALOG_FORMITEM * /*items*/);
 extern int dlg_ordinate(const char * /*s*/);
 extern void dlg_free_formitems(DIALOG_FORMITEM * /*items*/);
 
+/* guage.c */
+extern void * dlg_allocate_gauge(const char * /* title */, const char * /* cprompt */, int /* height */, int /* width */, int /* percent */);
+extern void dlg_free_gauge(void * /* objptr */);
+extern void dlg_update_gauge(void * /* objptr */, int /* percent */);
+
 /* inputstr.c */
 extern bool dlg_edit_string(char * /*string*/, int * /*offset*/, int /*key*/, int /*fkey*/, bool /*force*/);
 extern const int * dlg_index_columns(const char * /*string*/);
@@ -630,6 +666,10 @@ extern int dlg_find_index(const int * /*list*/, int  /*limit*/, int /*to_find*/)
 extern int dlg_limit_columns(const char * /*string*/, int /*limit*/, int /*offset*/);
 extern void dlg_show_string(WINDOW * /*win*/, const char * /*string*/, int /*offset*/, chtype /*attr*/, int /*y_base*/, int /*x_base*/, int /*x_last*/, bool /*hidden*/, bool /*force*/);
 
+/* menubox.c */
+extern int dlg_dummy_menutext(DIALOG_LISTITEM * /*items*/, int /*current*/, char * /*newtext*/);
+extern int dlg_renamed_menutext(DIALOG_LISTITEM * /*items*/, int /*current*/, char * /*newtext*/);
+
 /* rc.c */
 #ifdef HAVE_RC_FILE
 extern int dlg_parse_rc(void);
@@ -657,15 +697,17 @@ extern char * dlg_strempty(void);
 extern chtype dlg_asciibox(chtype /*ch*/);
 extern chtype dlg_boxchar(chtype /*ch*/);
 extern chtype dlg_get_attrs(WINDOW * /*win*/);
-extern const char * dlg_print_line(WINDOW */*win*/, chtype */*attr*/, const char */*prompt*/, int /*lm*/, int /*rm*/, int */*x*/);
+extern const char * dlg_print_line(WINDOW * /*win*/, chtype * /*attr*/, const char * /*prompt*/, int /*lm*/, int /*rm*/, int * /*x*/);
 extern int dlg_box_x_ordinate(int /*width*/);
 extern int dlg_box_y_ordinate(int /*height*/);
 extern int dlg_calc_list_width(int /*item_no*/, DIALOG_LISTITEM * /*items*/);
 extern int dlg_calc_listw(int /*item_no*/, char ** /*items*/, int /*group*/);
-extern int dlg_check_scrolled(int /* key */, int /* last */, int /* page */, bool */* show */, int */* offset */);
+extern int dlg_check_scrolled(int /* key */, int /* last */, int /* page */, bool * /* show */, int * /* offset */);
+extern int dlg_count_real_columns(const char * /*text*/);
 extern int dlg_default_item(char ** /*items*/, int /*llen*/);
 extern int dlg_default_listitem(DIALOG_LISTITEM * /*items*/);
 extern int dlg_defaultno_button(void);
+extern int dlg_default_button(void);
 extern int dlg_max_input(int /*max_len*/);
 extern int dlg_print_scrolled(WINDOW * /* win */, const char * /* prompt */, int /* offset */, int /* height */, int /* width */, int /* pauseopt */);
 extern void dlg_add_quoted(char * /*string*/);
@@ -683,7 +725,9 @@ extern void dlg_ctl_size(int /*height*/, int /*width*/);
 extern void dlg_del_window(WINDOW * /*win*/);
 extern void dlg_does_output(void);
 extern void dlg_draw_bottom_box(WINDOW * /*win*/);
+extern void dlg_draw_bottom_box2(WINDOW * /*win*/, chtype /*on_left*/, chtype /*on_right*/, chtype /*on_inside*/);
 extern void dlg_draw_box(WINDOW * /*win*/, int /*y*/, int /*x*/, int /*height*/, int /*width*/, chtype /*boxchar*/, chtype /*borderchar*/);
+extern void dlg_draw_box2(WINDOW * /*win*/, int /*y*/, int /*x*/, int /*height*/, int /*width*/, chtype /*boxchar*/, chtype /*borderchar*/, chtype /*borderchar2*/);
 extern void dlg_draw_title(WINDOW *win, const char *title);
 extern void dlg_exit(int /*code*/) GCC_NORETURN;
 extern void dlg_item_help(const char * /*txt*/);
@@ -699,11 +743,7 @@ extern void dlg_trim_string(char * /*src*/);
 extern void end_dialog(void);
 extern void init_dialog(FILE * /*input*/, FILE * /*output*/);
 
-extern void dlg_exiterr(const char *, ...) GCC_NORETURN
-#if defined(__GNUC__) && !defined(printf)
-__attribute__((format(printf,1,2)))
-#endif
-;
+extern void dlg_exiterr(const char *, ...) GCC_NORETURN GCC_PRINTFLIKE(1,2);
 
 #ifdef HAVE_COLOR
 extern chtype dlg_color_pair(int /*foreground*/, int /*background*/);
@@ -720,11 +760,7 @@ extern int dlg_strcmp(const char * /*a*/, const char * /*b*/);
 
 #ifdef HAVE_DLG_TRACE
 #define DLG_TRACE(params) dlg_trace_msg params
-extern void dlg_trace_msg(const char *fmt, ...)
-#ifdef GCC_PRINTF
-    __attribute__((format(printf,1,2)))
-#endif
-;
+extern void dlg_trace_msg(const char *fmt, ...) GCC_PRINTFLIKE(1,2);
 extern void dlg_trace_win(WINDOW * /*win*/);
 extern void dlg_trace_chr(int /*ch*/, int /*fkey*/);
 extern void dlg_trace(const char * /*fname*/);
@@ -739,6 +775,12 @@ extern void dlg_trace(const char * /*fname*/);
 extern void dlg_move_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 #endif
 
+/*
+ * Normally "enter" means "ok".  Use this macro to handle the explicit
+ * check for DLGK_ENTER:
+ */
+#define dlg_enter_buttoncode(code) (dialog_vars.nook ? DLG_EXIT_OK : dlg_ok_buttoncode(code))
+
 /*
  * The following stuff is needed for mouse support
  */
